import platform
import sys
from linkwiz.install import install, uninstall


def main():
    """Entry point of the program."""

    if platform.system() != "Linux":
        print("LinkWiz is only supported on Linux.")
        return

    if len(sys.argv) != 2:
        print("Usage: linkwiz [install | uninstall | <url>]")
        return

    arg = sys.argv[1]

    if arg == "install":
        install(sys.argv[0])
    elif arg == "uninstall":
        uninstall()
    else:
        from linkwiz.core import process_url

        process_url(arg)


if __name__ == "__main__":
    main()



import tkinter as tk
from urllib.parse import urlparse
from linkwiz.config import config
from typing import Dict
import logging


class LinkwizGUI:

    def __init__(self, browsers: Dict[str, str], url: str):
        self.url = url
        self.hostname = urlparse(url).netloc
        self.browsers = browsers

        self.root = tk.Tk()
        self.root.title("LinkWiz")
        self.root.resizable(False, False)
        self.root.wm_minsize(width=200, height=0)
        self.root.configure(bg="white")

        self.frame = tk.Frame(self.root, bg="white")
        self.frame.pack(pady=5)

        self.buttons = []
        self._create_widgets()
        self._bind_key_events()
        self._center_window()

    def _center_window(self):
        """Center the window on the screen."""
        window_width = self.root.winfo_reqwidth()
        window_height = self.root.winfo_reqheight()
        screen_width = self.root.winfo_screenwidth()
        screen_height = self.root.winfo_screenheight()
        x = max(0, (screen_width - window_width) // 2)
        y = max(0, (screen_height - window_height) // 2)
        self.root.geometry(f"+{x}+{y}")

    def _create_widgets(self):
        """Create widgets."""
        self._create_buttons()
        self._create_remember_check()

    def _create_buttons(self):
        """Create buttons for each browser."""
        for i, (browser_name, _) in enumerate(self.browsers.items()):
            button_text = f"{i+1}. {browser_name}"
            button = CustomButton(
                self.frame,
                text=button_text,
                command=lambda idx=i: self.get_launch_cmd(idx),
                anchor="w",
            )
            button.pack(fill=tk.X, padx=10, pady=5)
            self.buttons.append(button)

    def _create_remember_check(self):
        """Create 'Remember' checkbox."""
        self.remember = tk.BooleanVar()
        self.remember_check = CustomCheckbutton(
            self.root, text="Remember", variable=self.remember
        )
        self.remember_check.pack(anchor=tk.W, padx=10)

    def _bind_key_events(self):
        """Bind key press events."""
        try:
            self.root.bind("<Key>", self.on_key_pressed)
        except Exception as e:
            logging.error(f"Error binding key press: {e}")

    def on_key_pressed(self, event: tk.Event) -> None:
        """Handle key press events."""
        try:
            if event.char.isdigit():
                index = int(event.char) - 1
                if 0 <= index < len(self.browsers):
                    self.get_launch_cmd(index)
            elif event.char.lower() == "r":
                self.remember.set(not self.remember.get())
            elif event.char.lower() == "q" or event.char == "\x1b":
                self.root.destroy()
        except Exception as e:
            logging.error(f"Error handling key press: {e}")

    def get_launch_cmd(self, index):
        """Opens the selected browser with the given URL."""
        selected_browser_name = list(self.browsers.keys())[index]
        selected_browser = self.browsers[selected_browser_name]
        if self.remember.get():
            config.add_rules(self.hostname, selected_browser_name)
        self.result = selected_browser, self.url
        self.root.destroy()

    def run(self):
        """Run the application."""
        self.root.mainloop()
        return self.result


class CustomButton(tk.Button):
    def __init__(self, master=None, **kwargs):
        super().__init__(master, **kwargs)
        self.config(
            bg="white",
            fg="black",
            bd=2,
            relief="solid",
            activeforeground="black",
            padx=10,
            pady=5,
            cursor="hand2",
        )


class CustomCheckbutton(tk.Checkbutton):
    def __init__(self, master=None, **kwargs):
        super().__init__(master, **kwargs)
        self.config(
            bg="white",
            cursor="hand2",
            indicatoron=20,
        )



from pathlib import Path
import subprocess
from typing import List, Union
import logging

def exec_field_to_cmds(exe: Union[Path, str], link: str) -> List[str]:
    """
    Convert the executable field to a list of commands.
    >>> exec_field_to_cmds("firefox %u", "https://example.com")
    ['firefox', 'https://example.com']
    >>> exec_field_to_cmds(Path("firefox"), "https://example.com")
    ['firefox', 'https://example.com']
    >>> exec_field_to_cmds("firefox %U", "https://example.com")
    ['firefox', 'https://example.com']
    >>> exec_field_to_cmds("firefox", "https://example.com")
    ['firefox', 'https://example.com']
    """
    assert isinstance(exe, (Path, str)), "exe must be Path or str"
    exe_str = str(exe) if isinstance(exe, Path) else exe

    cmd = exe_str.replace("%u", link).replace("%U", link)
    if link not in cmd:
        cmd += f" {link}"

    return cmd.split()


def launch_browser(exe: Union[Path, str], link: str) -> None:
    """
    Open the link using the specified executable.
    """
    cmd = exec_field_to_cmds(exe, link)
    try:
        subprocess.Popen(
            cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=False
        )
        logging.info(f"Opened link: {link}")
    except (FileNotFoundError, PermissionError) as e:
        logging.error(f"Error opening link: {e}")
    exit()



from pathlib import Path
import subprocess
from typing import Dict, List
from xdg import DesktopEntry
from linkwiz.config import config

APPNAME: str = "LinkWiz"
HTTP_HANDLER: str = "x-scheme-handler/http"

DESKTOP_PATHS = [
    Path("/usr/share/applications/"),
    Path.home() / ".local/share/applications/",
]


def get_browsers() -> Dict[str, Path]:
    """Get the name and exec path of browsers."""
    try:
        installed_browsers = []
        if config.main.get("auto_find_browsers", True):
            output = subprocess.check_output(["gio", "mime", HTTP_HANDLER], text=True)
            installed_browsers = (
                output.split("Recommended applications:")[-1].strip().split("\n")
            )
            installed_browsers = [app.strip() for app in installed_browsers]

            own_desktop = f"{APPNAME.lower()}.desktop"

            if own_desktop in installed_browsers:
                installed_browsers.remove(own_desktop)

        return get_browser_exec(installed_browsers)
    except subprocess.CalledProcessError:
        raise ("Error: Unable to retrieve installed browsers.")


def get_browser_exec(browsers_desktop: List[str]) -> Dict[str, Path]:
    """Get the exec path of installed browsers."""
    installed_browsers: Dict[str, Path] = {}
    for path in DESKTOP_PATHS:
        if not path.exists():
            continue
        for entry in path.glob("*.desktop"):
            if entry.name not in browsers_desktop:
                continue
            desktop_entry = DesktopEntry.DesktopEntry(str(entry))
            name: str = desktop_entry.getName()
            execpath: str = desktop_entry.getExec()
            installed_browsers[name] = Path(execpath)

    installed_browsers.update(config.browsers)

    return installed_browsers



from xdg import DesktopEntry, BaseDirectory
from pathlib import Path
import subprocess
import shlex

DESKTOP_FILENAME = "linkwiz.desktop"
DESKTOP_PATH = Path(BaseDirectory.xdg_data_home) / "applications" / DESKTOP_FILENAME


def install(script_path: str):
    create_linkwiz_desktop_entry(script_path)
    mime_types = ["x-scheme-handler/http", "x-scheme-handler/https"]
    for mime_type in mime_types:
        set_default_app_for_mime_type(mime_type)
    print("Installed")


def create_linkwiz_desktop_entry(script_path: str):
    desktop = DesktopEntry.DesktopEntry(DESKTOP_PATH)
    desktop.set("Name", "Linkwiz")
    desktop.set("Type", "Application")
    desktop.set("MimeType", "x-scheme-handler/http;x-scheme-handler/https;")
    desktop.set("Categories", "Network;")
    desktop.set("NoDisplay", "true")
    desktop.set("Exec", script_path + " %u")
    desktop.write()


def set_default_app_for_mime_type(mime_type: str):
    cmd = ["xdg-mime", "default", DESKTOP_FILENAME, mime_type]
    mime_type_quoted = shlex.quote(mime_type)
    subprocess.run(cmd + [mime_type_quoted], check=True)


def uninstall():
    if DESKTOP_PATH.exists():
        DESKTOP_PATH.unlink()
        print("Uninstalled")
    else:
        print("linkwiz is not installed.")
